import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="TimeSeriesPeriod")


@attr.s(auto_attribs=True)
class TimeSeriesPeriod:
    """
    Attributes:
        start (Union[Unset, datetime.datetime]):
        end (Union[Unset, datetime.datetime]):
    """

    start: Union[Unset, datetime.datetime] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        time_series_period = cls(
            start=start,
            end=end,
        )

        return time_series_period



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.day_data_by_hour_transfer import DayDataByHourTransfer
from ..types import UNSET, Unset

T = TypeVar("T", bound="WeekDataTransfere")


@attr.s(auto_attribs=True)
class WeekDataTransfere:
    """
    Attributes:
        discriminator (str):
        monday_data (Union[Unset, None, DayDataByHourTransfer]):
        tuesday_data (Union[Unset, None, DayDataByHourTransfer]):
        wednesday_data (Union[Unset, None, DayDataByHourTransfer]):
        thursday_data (Union[Unset, None, DayDataByHourTransfer]):
        friday_data (Union[Unset, None, DayDataByHourTransfer]):
        saturday_data (Union[Unset, None, DayDataByHourTransfer]):
        sunday_data (Union[Unset, None, DayDataByHourTransfer]):
    """

    discriminator: str
    monday_data: Union[Unset, None, DayDataByHourTransfer] = UNSET
    tuesday_data: Union[Unset, None, DayDataByHourTransfer] = UNSET
    wednesday_data: Union[Unset, None, DayDataByHourTransfer] = UNSET
    thursday_data: Union[Unset, None, DayDataByHourTransfer] = UNSET
    friday_data: Union[Unset, None, DayDataByHourTransfer] = UNSET
    saturday_data: Union[Unset, None, DayDataByHourTransfer] = UNSET
    sunday_data: Union[Unset, None, DayDataByHourTransfer] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        discriminator = self.discriminator
        monday_data: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.monday_data, Unset):
            monday_data = self.monday_data.to_dict() if self.monday_data else None

        tuesday_data: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.tuesday_data, Unset):
            tuesday_data = self.tuesday_data.to_dict() if self.tuesday_data else None

        wednesday_data: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.wednesday_data, Unset):
            wednesday_data = self.wednesday_data.to_dict() if self.wednesday_data else None

        thursday_data: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.thursday_data, Unset):
            thursday_data = self.thursday_data.to_dict() if self.thursday_data else None

        friday_data: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.friday_data, Unset):
            friday_data = self.friday_data.to_dict() if self.friday_data else None

        saturday_data: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.saturday_data, Unset):
            saturday_data = self.saturday_data.to_dict() if self.saturday_data else None

        sunday_data: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.sunday_data, Unset):
            sunday_data = self.sunday_data.to_dict() if self.sunday_data else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "discriminator": discriminator,
            }
        )
        if monday_data is not UNSET:
            field_dict["mondayData"] = monday_data
        if tuesday_data is not UNSET:
            field_dict["tuesdayData"] = tuesday_data
        if wednesday_data is not UNSET:
            field_dict["wednesdayData"] = wednesday_data
        if thursday_data is not UNSET:
            field_dict["thursdayData"] = thursday_data
        if friday_data is not UNSET:
            field_dict["fridayData"] = friday_data
        if saturday_data is not UNSET:
            field_dict["saturdayData"] = saturday_data
        if sunday_data is not UNSET:
            field_dict["sundayData"] = sunday_data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discriminator = d.pop("discriminator")

        _monday_data = d.pop("mondayData", UNSET)
        monday_data: Union[Unset, None, DayDataByHourTransfer]
        if _monday_data is None:
            monday_data = None
        elif isinstance(_monday_data, Unset):
            monday_data = UNSET
        else:
            monday_data = DayDataByHourTransfer.from_dict(_monday_data)

        _tuesday_data = d.pop("tuesdayData", UNSET)
        tuesday_data: Union[Unset, None, DayDataByHourTransfer]
        if _tuesday_data is None:
            tuesday_data = None
        elif isinstance(_tuesday_data, Unset):
            tuesday_data = UNSET
        else:
            tuesday_data = DayDataByHourTransfer.from_dict(_tuesday_data)

        _wednesday_data = d.pop("wednesdayData", UNSET)
        wednesday_data: Union[Unset, None, DayDataByHourTransfer]
        if _wednesday_data is None:
            wednesday_data = None
        elif isinstance(_wednesday_data, Unset):
            wednesday_data = UNSET
        else:
            wednesday_data = DayDataByHourTransfer.from_dict(_wednesday_data)

        _thursday_data = d.pop("thursdayData", UNSET)
        thursday_data: Union[Unset, None, DayDataByHourTransfer]
        if _thursday_data is None:
            thursday_data = None
        elif isinstance(_thursday_data, Unset):
            thursday_data = UNSET
        else:
            thursday_data = DayDataByHourTransfer.from_dict(_thursday_data)

        _friday_data = d.pop("fridayData", UNSET)
        friday_data: Union[Unset, None, DayDataByHourTransfer]
        if _friday_data is None:
            friday_data = None
        elif isinstance(_friday_data, Unset):
            friday_data = UNSET
        else:
            friday_data = DayDataByHourTransfer.from_dict(_friday_data)

        _saturday_data = d.pop("saturdayData", UNSET)
        saturday_data: Union[Unset, None, DayDataByHourTransfer]
        if _saturday_data is None:
            saturday_data = None
        elif isinstance(_saturday_data, Unset):
            saturday_data = UNSET
        else:
            saturday_data = DayDataByHourTransfer.from_dict(_saturday_data)

        _sunday_data = d.pop("sundayData", UNSET)
        sunday_data: Union[Unset, None, DayDataByHourTransfer]
        if _sunday_data is None:
            sunday_data = None
        elif isinstance(_sunday_data, Unset):
            sunday_data = UNSET
        else:
            sunday_data = DayDataByHourTransfer.from_dict(_sunday_data)

        week_data_transfere = cls(
            discriminator=discriminator,
            monday_data=monday_data,
            tuesday_data=tuesday_data,
            wednesday_data=wednesday_data,
            thursday_data=thursday_data,
            friday_data=friday_data,
            saturday_data=saturday_data,
            sunday_data=sunday_data,
        )

        week_data_transfere.additional_properties = d
        return week_data_transfere

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EnDataPointStateDTO(str, Enum):
    DRAFT = "Draft"
    INPROGRESS = "InProgress"
    READY = "Ready"
    ERROR = "Error"

    def __str__(self) -> str:
        return str(self.value)



import json
import uuid
from typing import Any, List, Optional, Type, TypeVar, Union

import pandas as pd
from pandas import DataFrame
from structlog import get_logger

from data_point_client import AuthenticatedClient
from data_point_client.api.data_point import (
    data_point_append_execution_result_data,
    data_point_append_time_series_data,
    data_point_create_data_point,
    data_point_get_data,
    data_point_get_data_point,
    data_point_update_constant_data,
    data_point_update_time_series_data,
    data_point_update_week_period_data,
)
from data_point_client.models import GetConstantResponse, GetSeriesResponse
from data_point_client.models.append_execution_result_data_request import AppendExecutionResultDataRequest
from data_point_client.models.append_time_series_request import AppendTimeSeriesRequest
from data_point_client.models.data_point_request import DataPointRequest
from data_point_client.models.en_data_point_existence_dto import EnDataPointExistenceDTO
from data_point_client.models.get_data_request import GetDataRequest
from data_point_client.models.problem_details import ProblemDetails
from data_point_client.models.sub_series_request import SubSeriesRequest
from data_point_client.models.sub_series_request_values import SubSeriesRequestValues
from data_point_client.models.time_series_response_curve import TimeSeriesResponseCurve
from data_point_client.models.update_constant_data_request import UpdateConstantDataRequest
from data_point_client.models.update_time_series_request import UpdateTimeSeriesRequest
from data_point_client.models.update_week_period_request import UpdateWeekPeriodRequest
from data_point_client.models.week_data_transfere import WeekDataTransfere
from data_point_client.types import Response, Unset
from nista_library.nista_connetion import NistaConnection

log = get_logger()

# pylint: disable=C0103
T = TypeVar("T", bound="NistaDataPoint")


class NistaDataPoint:
    DATE_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
    DATE_NAME = "Date"
    VALUE_NAME = "Value"

    @classmethod
    def create_new(cls: Type[T], connection: NistaConnection, name: str, tags: List[str]) -> Union[T, None]:
        token = connection.get_access_token()
        client = AuthenticatedClient(
            base_url=connection.datapoint_base_url, token=token, verify_ssl=connection.verify_ssl
        )

        new_id = uuid.uuid4()
        request = DataPointRequest(name=name, description="", existence=EnDataPointExistenceDTO.FULL, tags=tags)

        response = data_point_create_data_point.sync(
            client=client, data_point_id=str(new_id), workspace_id=connection.workspace_id, json_body=request
        )

        if isinstance(response, ProblemDetails):
            log.error(response)
            return None

        data_point = cls(connection=connection, data_point_id=new_id, name=name)
        return data_point

    def __init__(self, connection: NistaConnection, data_point_id: uuid.UUID, name: Optional[str] = None):
        self.connection = connection
        self.data_point_id = data_point_id
        if name is None:
            self._load_details()
        else:
            self.name = name

    def __str__(self):
        return "NistaDataPoint " + self.data_point_id + " with name " + self.name

    def _load_details(self):
        token = self.connection.get_access_token()
        client = AuthenticatedClient(
            base_url=self.connection.datapoint_base_url, token=token, verify_ssl=self.connection.verify_ssl
        )

        data_point = data_point_get_data_point.sync(
            client=client,
            data_point_id=self.data_point_id,
            workspace_id=self.connection.workspace_id,
        )

        if data_point is None:
            raise Exception("Cannot load Datapoint")
        self.name = data_point.name

    # pylint: disable=too-many-arguments
    def get_data_point_data(
        self,
        request: GetDataRequest,
        post_fix: bool = False,
    ) -> Union[List[DataFrame], float, Unset, None]:
        token = self.connection.get_access_token()
        client = AuthenticatedClient(
            base_url=self.connection.datapoint_base_url, token=token, verify_ssl=self.connection.verify_ssl, timeout=30
        )

        byte_content = data_point_get_data.sync_detailed(
            client=client,
            data_point_id=str(self.data_point_id),
            workspace_id=self.connection.workspace_id,
            json_body=request,
        ).content

        log.debug("Received Response from nista.io", content=byte_content)

        content_text = byte_content.decode("utf-8")
        jscon_content = json.loads(content_text)

        content_type = jscon_content["discriminator"]

        if content_type == "GetSeriesResponse":

            series_response = GetSeriesResponse.from_dict(jscon_content)
            curves = series_response.curves

            if isinstance(curves, list):
                data_frames = []
                # pylint: disable=E1133
                for curve in curves:
                    # pylint: enable=E1133
                    curve_dict = curve.curve
                    if isinstance(curve_dict, TimeSeriesResponseCurve):
                        data_frame = self._from_time_frames(time_frames=curve_dict.to_dict(), post_fix=post_fix)
                        data_frames.append(data_frame)

                return data_frames

        if content_type == "GetConstantResponse":
            constant_response = GetConstantResponse.from_dict(jscon_content)
            if constant_response is not None:
                return constant_response.value

        return None

    def append_data_point_data(
        self,
        data: Union[List[DataFrame], float],
        unit: Union[str, None],
    ) -> Union[Response[Union[Any, ProblemDetails]], None]:
        token = self.connection.get_access_token()
        client = AuthenticatedClient(
            base_url=self.connection.datapoint_base_url, token=token, verify_ssl=self.connection.verify_ssl
        )

        if isinstance(data, list):
            sub_series: List[SubSeriesRequest] = []
            for data_frame in data:
                data_dict = self._to_dict(data_frame)
                value = SubSeriesRequestValues.from_dict(src_dict=data_dict)
                request = SubSeriesRequest(values=value)
                sub_series.append(request)

            append_request = AppendTimeSeriesRequest(sub_series=sub_series, unit=unit)

            return data_point_append_time_series_data.sync_detailed(
                workspace_id=self.connection.workspace_id,
                client=client,
                data_point_id=str(self.data_point_id),
                json_body=append_request,
            )

        return None

    def append_data_point_result_parts(
        self,
        data: Union[List[DataFrame], float],
        unit: Union[str, None],
        execution_id: uuid.UUID,
    ) -> Union[Response[Union[Any, ProblemDetails]], None]:
        token = self.connection.get_access_token()
        client = AuthenticatedClient(
            base_url=self.connection.datapoint_base_url, token=token, verify_ssl=self.connection.verify_ssl
        )

        if isinstance(data, list):
            sub_series: List[SubSeriesRequest] = []
            for data_frame in data:
                data_dict = self._to_dict(data_frame)
                value = SubSeriesRequestValues.from_dict(src_dict=data_dict)
                request = SubSeriesRequest(values=value)
                sub_series.append(request)

            append_request = AppendExecutionResultDataRequest(
                sub_series=sub_series, unit=unit, execution_id=str(execution_id)
            )

            return data_point_append_execution_result_data.sync_detailed(
                workspace_id=self.connection.workspace_id,
                client=client,
                data_point_id=str(self.data_point_id),
                json_body=append_request,
            )

        return None

    def set_data_point_data(
        self,
        data: Union[List[DataFrame], float, WeekDataTransfere],
        unit: Union[str, None],
        execution_id: Union[str, None],
    ) -> Union[Response[Union[Any, ProblemDetails]], None]:
        token = self.connection.get_access_token()
        client = AuthenticatedClient(
            base_url=self.connection.datapoint_base_url, token=token, verify_ssl=self.connection.verify_ssl
        )

        if isinstance(data, WeekDataTransfere):
            week_update_request = UpdateWeekPeriodRequest(execution_id=execution_id, unit=unit, week_data=data)
            return data_point_update_week_period_data.sync_detailed(
                workspace_id=self.connection.workspace_id,
                client=client,
                data_point_id=str(self.data_point_id),
                json_body=week_update_request,
            )

        if isinstance(data, list):
            sub_series: List[SubSeriesRequest] = []
            for data_frame in data:
                data_dict = self._to_dict(data_frame)
                value = SubSeriesRequestValues.from_dict(src_dict=data_dict)
                request = SubSeriesRequest(values=value)
                sub_series.append(request)

            timeseries_update_request = UpdateTimeSeriesRequest(
                sub_series=sub_series, unit=unit, execution_id=execution_id
            )

            return data_point_update_time_series_data.sync_detailed(
                workspace_id=self.connection.workspace_id,
                client=client,
                data_point_id=str(self.data_point_id),
                json_body=timeseries_update_request,
            )

        if isinstance(data, float):
            constant_request = UpdateConstantDataRequest(
                value=data,
                unit=unit,
                execution_id=execution_id,
            )

            return data_point_update_constant_data.sync_detailed(
                workspace_id=self.connection.workspace_id,
                client=client,
                data_point_id=str(self.data_point_id),
                json_body=constant_request,
            )

        return None

    def _to_dict(self, data_frame: DataFrame):
        result = {}
        dct = data_frame.to_dict()[self.VALUE_NAME]
        for key in dct.keys():
            value = dct[key]
            key_string = key.strftime(self.DATE_FORMAT)
            result[key_string] = value
        return result

    def _from_time_frames(self, time_frames: dict, post_fix: bool, date_format: str = DATE_FORMAT) -> DataFrame:

        if not isinstance(time_frames, dict):
            raise TypeError

        value_column_name = self.name

        if value_column_name is None:
            value_column_name = self.VALUE_NAME

        if post_fix:
            value_column_name = value_column_name + "_nista.io_" + str(self.data_point_id)

        log.debug("Reading data as Pandas DataFrame")

        data_record = []
        for date in time_frames:
            value = time_frames[date]
            data_record.append({self.DATE_NAME: date, value_column_name: value})

        data_frame = pd.DataFrame.from_records(data_record, columns=[self.DATE_NAME, value_column_name])

        data_frame[self.DATE_NAME] = pd.to_datetime(data_frame[self.DATE_NAME], format=date_format)

        data_frame[value_column_name] = pd.to_numeric(data_frame[value_column_name])

        data_frame = data_frame.set_index(data_frame[self.DATE_NAME])
        data_frame = data_frame.drop([self.DATE_NAME], axis=1)

        return data_frame



from oauth2_client.credentials_manager import CredentialManager
from structlog import get_logger

log = get_logger()


class NistaCredentialManager(CredentialManager):
    def __init__(self, service_information, proxies=None):
        super().__init__(service_information, proxies)
        self.id_token = None
        self.access_token = None
        self.refresh_token = None

    def _process_token_response(self, token_response, refresh_token_mandatory):
        log.info("Token has been received.")

        access_token = token_response.get("access_token")

        id_token = token_response.get("id_token")
        if id_token is not None:
            self.id_token = id_token

        super()._process_token_response(token_response, refresh_token_mandatory)
        self.access_token = access_token
        self.refresh_token = token_response.get("refresh_token")
        log.info("Successfully logged in")



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.data_point_data_response import DataPointDataResponse
from ..models.en_data_bucket_state import EnDataBucketState
from ..models.gnista_unit_response import GnistaUnitResponse
from ..types import UNSET, Unset

T = TypeVar("T", bound="DataBucketBase")


@attr.s(auto_attribs=True)
class DataBucketBase:
    """
    Attributes:
        discriminator (str):
        version (Union[Unset, int]):
        created (Union[Unset, datetime.datetime]):
        status (Union[Unset, EnDataBucketState]):
        violations (Union[Unset, int]):
        unit (Union[Unset, None, str]):
        gnista_unit (Union[Unset, None, GnistaUnitResponse]):
        error_details (Union[Unset, None, str]):
        warning_details (Union[Unset, None, str]):
        available_data_points (Union[Unset, None, List[DataPointDataResponse]]):
    """

    discriminator: str
    version: Union[Unset, int] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, EnDataBucketState] = UNSET
    violations: Union[Unset, int] = UNSET
    unit: Union[Unset, None, str] = UNSET
    gnista_unit: Union[Unset, None, GnistaUnitResponse] = UNSET
    error_details: Union[Unset, None, str] = UNSET
    warning_details: Union[Unset, None, str] = UNSET
    available_data_points: Union[Unset, None, List[DataPointDataResponse]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        discriminator = self.discriminator
        version = self.version
        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        violations = self.violations
        unit = self.unit
        gnista_unit: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.gnista_unit, Unset):
            gnista_unit = self.gnista_unit.to_dict() if self.gnista_unit else None

        error_details = self.error_details
        warning_details = self.warning_details
        available_data_points: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.available_data_points, Unset):
            if self.available_data_points is None:
                available_data_points = None
            else:
                available_data_points = []
                for available_data_points_item_data in self.available_data_points:
                    available_data_points_item = available_data_points_item_data.to_dict()

                    available_data_points.append(available_data_points_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "discriminator": discriminator,
            }
        )
        if version is not UNSET:
            field_dict["version"] = version
        if created is not UNSET:
            field_dict["created"] = created
        if status is not UNSET:
            field_dict["status"] = status
        if violations is not UNSET:
            field_dict["violations"] = violations
        if unit is not UNSET:
            field_dict["unit"] = unit
        if gnista_unit is not UNSET:
            field_dict["gnistaUnit"] = gnista_unit
        if error_details is not UNSET:
            field_dict["errorDetails"] = error_details
        if warning_details is not UNSET:
            field_dict["warningDetails"] = warning_details
        if available_data_points is not UNSET:
            field_dict["availableDataPoints"] = available_data_points

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discriminator = d.pop("discriminator")

        version = d.pop("version", UNSET)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset):
            created = UNSET
        else:
            created = isoparse(_created)

        _status = d.pop("status", UNSET)
        status: Union[Unset, EnDataBucketState]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = EnDataBucketState(_status)

        violations = d.pop("violations", UNSET)

        unit = d.pop("unit", UNSET)

        _gnista_unit = d.pop("gnistaUnit", UNSET)
        gnista_unit: Union[Unset, None, GnistaUnitResponse]
        if _gnista_unit is None:
            gnista_unit = None
        elif isinstance(_gnista_unit, Unset):
            gnista_unit = UNSET
        else:
            gnista_unit = GnistaUnitResponse.from_dict(_gnista_unit)

        error_details = d.pop("errorDetails", UNSET)

        warning_details = d.pop("warningDetails", UNSET)

        available_data_points = []
        _available_data_points = d.pop("availableDataPoints", UNSET)
        for available_data_points_item_data in _available_data_points or []:
            available_data_points_item = DataPointDataResponse.from_dict(available_data_points_item_data)

            available_data_points.append(available_data_points_item)

        data_bucket_base = cls(
            discriminator=discriminator,
            version=version,
            created=created,
            status=status,
            violations=violations,
            unit=unit,
            gnista_unit=gnista_unit,
            error_details=error_details,
            warning_details=warning_details,
            available_data_points=available_data_points,
        )

        return data_bucket_base



""" Contains all the data models used in inputs/outputs """

from .append_execution_result_data_request import AppendExecutionResultDataRequest
from .append_time_series_request import AppendTimeSeriesRequest
from .calculation_origin import CalculationOrigin
from .constant_data_bucket import ConstantDataBucket
from .constant_data_point_data import ConstantDataPointData
from .data_bucket_base import DataBucketBase
from .data_export_request import DataExportRequest
from .data_point_data_base import DataPointDataBase
from .data_point_data_response import DataPointDataResponse
from .data_point_origin import DataPointOrigin
from .data_point_request import DataPointRequest
from .data_point_response_base import DataPointResponseBase
from .day_data_by_hour_transfer import DayDataByHourTransfer
from .en_data_bucket_state import EnDataBucketState
from .en_data_point_existence_dto import EnDataPointExistenceDTO
from .en_data_point_state_dto import EnDataPointStateDTO
from .en_data_point_status import EnDataPointStatus
from .en_data_point_type import EnDataPointType
from .en_operator import EnOperator
from .file_origin import FileOrigin
from .get_constant_response import GetConstantResponse
from .get_data_request import GetDataRequest
from .get_data_response import GetDataResponse
from .get_series_response import GetSeriesResponse
from .gnista_unit_response import GnistaUnitResponse
from .problem_details import ProblemDetails
from .problem_details_extensions import ProblemDetailsExtensions
from .remote_origin import RemoteOrigin
from .rule import Rule
from .series_data_bucket import SeriesDataBucket
from .sub_series_request import SubSeriesRequest
from .sub_series_request_values import SubSeriesRequestValues
from .time_series_period import TimeSeriesPeriod
from .time_series_response import TimeSeriesResponse
from .time_series_response_curve import TimeSeriesResponseCurve
from .update_constant_data_request import UpdateConstantDataRequest
from .update_day_period_request import UpdateDayPeriodRequest
from .update_time_series_request import UpdateTimeSeriesRequest
from .update_week_period_request import UpdateWeekPeriodRequest
from .week_data_transfere import WeekDataTransfere



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FileOrigin")


@attr.s(auto_attribs=True)
class FileOrigin:
    """
    Attributes:
        discriminator (str):
        file_id (Union[Unset, None, str]):
        file_name (Union[Unset, None, str]):
        task_config_id (Union[Unset, None, str]):
    """

    discriminator: str
    file_id: Union[Unset, None, str] = UNSET
    file_name: Union[Unset, None, str] = UNSET
    task_config_id: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        discriminator = self.discriminator
        file_id = self.file_id
        file_name = self.file_name
        task_config_id = self.task_config_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "discriminator": discriminator,
            }
        )
        if file_id is not UNSET:
            field_dict["fileId"] = file_id
        if file_name is not UNSET:
            field_dict["fileName"] = file_name
        if task_config_id is not UNSET:
            field_dict["taskConfigId"] = task_config_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discriminator = d.pop("discriminator")

        file_id = d.pop("fileId", UNSET)

        file_name = d.pop("fileName", UNSET)

        task_config_id = d.pop("taskConfigId", UNSET)

        file_origin = cls(
            discriminator=discriminator,
            file_id=file_id,
            file_name=file_name,
            task_config_id=task_config_id,
        )

        file_origin.additional_properties = d
        return file_origin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EnDataPointType(str, Enum):
    IMPORTDATAPOINT = "ImportDataPoint"
    CALCULATIONDATAPOINT = "CalculationDataPoint"
    REMOTEDATAPOINT = "RemoteDataPoint"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.data_point_origin import DataPointOrigin
from ..models.en_data_point_existence_dto import EnDataPointExistenceDTO
from ..types import UNSET, Unset

T = TypeVar("T", bound="DataPointRequest")


@attr.s(auto_attribs=True)
class DataPointRequest:
    """
    Attributes:
        name (Union[Unset, None, str]):
        description (Union[Unset, None, str]):
        existence (Union[Unset, None, EnDataPointExistenceDTO]):
        tags (Union[Unset, None, List[str]]):
        origin (Union[Unset, None, DataPointOrigin]):
    """

    name: Union[Unset, None, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    existence: Union[Unset, None, EnDataPointExistenceDTO] = UNSET
    tags: Union[Unset, None, List[str]] = UNSET
    origin: Union[Unset, None, DataPointOrigin] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        description = self.description
        existence: Union[Unset, None, str] = UNSET
        if not isinstance(self.existence, Unset):
            existence = self.existence.value if self.existence else None

        tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            if self.tags is None:
                tags = None
            else:
                tags = self.tags

        origin: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.origin, Unset):
            origin = self.origin.to_dict() if self.origin else None

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if existence is not UNSET:
            field_dict["existence"] = existence
        if tags is not UNSET:
            field_dict["tags"] = tags
        if origin is not UNSET:
            field_dict["origin"] = origin

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        _existence = d.pop("existence", UNSET)
        existence: Union[Unset, None, EnDataPointExistenceDTO]
        if _existence is None:
            existence = None
        elif isinstance(_existence, Unset):
            existence = UNSET
        else:
            existence = EnDataPointExistenceDTO(_existence)

        tags = cast(List[str], d.pop("tags", UNSET))

        _origin = d.pop("origin", UNSET)
        origin: Union[Unset, None, DataPointOrigin]
        if _origin is None:
            origin = None
        elif isinstance(_origin, Unset):
            origin = UNSET
        else:
            origin = DataPointOrigin.from_dict(_origin)

        data_point_request = cls(
            name=name,
            description=description,
            existence=existence,
            tags=tags,
            origin=origin,
        )

        return data_point_request



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.sub_series_request import SubSeriesRequest
from ..types import UNSET, Unset

T = TypeVar("T", bound="AppendExecutionResultDataRequest")


@attr.s(auto_attribs=True)
class AppendExecutionResultDataRequest:
    """
    Attributes:
        sub_series (List[SubSeriesRequest]):
        unit (Union[Unset, None, str]):
        execution_id (Union[Unset, str]):
    """

    sub_series: List[SubSeriesRequest]
    unit: Union[Unset, None, str] = UNSET
    execution_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        sub_series = []
        for sub_series_item_data in self.sub_series:
            sub_series_item = sub_series_item_data.to_dict()

            sub_series.append(sub_series_item)

        unit = self.unit
        execution_id = self.execution_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "subSeries": sub_series,
            }
        )
        if unit is not UNSET:
            field_dict["unit"] = unit
        if execution_id is not UNSET:
            field_dict["executionId"] = execution_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        sub_series = []
        _sub_series = d.pop("subSeries")
        for sub_series_item_data in _sub_series:
            sub_series_item = SubSeriesRequest.from_dict(sub_series_item_data)

            sub_series.append(sub_series_item)

        unit = d.pop("unit", UNSET)

        execution_id = d.pop("executionId", UNSET)

        append_execution_result_data_request = cls(
            sub_series=sub_series,
            unit=unit,
            execution_id=execution_id,
        )

        return append_execution_result_data_request



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.sub_series_request_values import SubSeriesRequestValues
from ..types import UNSET, Unset

T = TypeVar("T", bound="SubSeriesRequest")


@attr.s(auto_attribs=True)
class SubSeriesRequest:
    """
    Attributes:
        values (Union[Unset, None, SubSeriesRequestValues]):
    """

    values: Union[Unset, None, SubSeriesRequestValues] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        values: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.values, Unset):
            values = self.values.to_dict() if self.values else None

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if values is not UNSET:
            field_dict["values"] = values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _values = d.pop("values", UNSET)
        values: Union[Unset, None, SubSeriesRequestValues]
        if _values is None:
            values = None
        elif isinstance(_values, Unset):
            values = UNSET
        else:
            values = SubSeriesRequestValues.from_dict(_values)

        sub_series_request = cls(
            values=values,
        )

        return sub_series_request



from enum import Enum


class EnDataPointExistenceDTO(str, Enum):
    FULL = "Full"
    LOW = "Low"
    HIDDEN = "Hidden"
    DELETED = "Deleted"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.week_data_transfere import WeekDataTransfere
from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateWeekPeriodRequest")


@attr.s(auto_attribs=True)
class UpdateWeekPeriodRequest:
    """
    Attributes:
        execution_id (Union[Unset, None, str]):
        week_data (Union[Unset, None, WeekDataTransfere]):
        unit (Union[Unset, None, str]):
    """

    execution_id: Union[Unset, None, str] = UNSET
    week_data: Union[Unset, None, WeekDataTransfere] = UNSET
    unit: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        execution_id = self.execution_id
        week_data: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.week_data, Unset):
            week_data = self.week_data.to_dict() if self.week_data else None

        unit = self.unit

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if execution_id is not UNSET:
            field_dict["executionId"] = execution_id
        if week_data is not UNSET:
            field_dict["weekData"] = week_data
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        execution_id = d.pop("executionId", UNSET)

        _week_data = d.pop("weekData", UNSET)
        week_data: Union[Unset, None, WeekDataTransfere]
        if _week_data is None:
            week_data = None
        elif isinstance(_week_data, Unset):
            week_data = UNSET
        else:
            week_data = WeekDataTransfere.from_dict(_week_data)

        unit = d.pop("unit", UNSET)

        update_week_period_request = cls(
            execution_id=execution_id,
            week_data=week_data,
            unit=unit,
        )

        return update_week_period_request



import uuid
from typing import Generator, List, Optional

from structlog import get_logger

from data_point_client import AuthenticatedClient
from data_point_client.api.data_point import data_point_get_data_points
from data_point_client.models import DataPointResponseBase, EnDataPointExistenceDTO, ProblemDetails
from nista_library.nista_connetion import NistaConnection
from nista_library.nista_data_point import NistaDataPoint

log = get_logger()


class NistaDataPoints:
    def __init__(self, connection: NistaConnection):
        self.connection = connection

    def get_data_point_list(self) -> Generator[NistaDataPoint, None, None]:
        token = self.connection.get_access_token()
        client = AuthenticatedClient(
            base_url=self.connection.datapoint_base_url, token=token, verify_ssl=self.connection.verify_ssl
        )

        data_point_list = data_point_get_data_points.sync(
            workspace_id=self.connection.workspace_id, client=client, existence=[EnDataPointExistenceDTO.FULL]
        )
        if isinstance(data_point_list, ProblemDetails):
            problems: ProblemDetails = data_point_list
            raise Exception(problems)

        if isinstance(data_point_list, list):
            list_of_data_points: List[DataPointResponseBase] = data_point_list

        for data_point in list_of_data_points:
            name: Optional[str] = None
            if isinstance(data_point.name, str):
                name = data_point.name

            nista_data_point = NistaDataPoint(
                connection=self.connection, data_point_id=uuid.UUID(data_point.data_point_id), name=name
            )
            yield nista_data_point



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GnistaUnitResponse")


@attr.s(auto_attribs=True)
class GnistaUnitResponse:
    """
    Attributes:
        name (Union[Unset, None, str]):
        physical_quantity (Union[Unset, None, str]):
    """

    name: Union[Unset, None, str] = UNSET
    physical_quantity: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        physical_quantity = self.physical_quantity

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if physical_quantity is not UNSET:
            field_dict["physicalQuantity"] = physical_quantity

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        physical_quantity = d.pop("physicalQuantity", UNSET)

        gnista_unit_response = cls(
            name=name,
            physical_quantity=physical_quantity,
        )

        return gnista_unit_response



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.problem_details_extensions import ProblemDetailsExtensions
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProblemDetails")


@attr.s(auto_attribs=True)
class ProblemDetails:
    """
    Attributes:
        type (Union[Unset, None, str]):
        title (Union[Unset, None, str]):
        status (Union[Unset, None, int]):
        detail (Union[Unset, None, str]):
        instance (Union[Unset, None, str]):
        extensions (Union[Unset, ProblemDetailsExtensions]):
    """

    type: Union[Unset, None, str] = UNSET
    title: Union[Unset, None, str] = UNSET
    status: Union[Unset, None, int] = UNSET
    detail: Union[Unset, None, str] = UNSET
    instance: Union[Unset, None, str] = UNSET
    extensions: Union[Unset, ProblemDetailsExtensions] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        title = self.title
        status = self.status
        detail = self.detail
        instance = self.instance
        extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.extensions, Unset):
            extensions = self.extensions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if title is not UNSET:
            field_dict["title"] = title
        if status is not UNSET:
            field_dict["status"] = status
        if detail is not UNSET:
            field_dict["detail"] = detail
        if instance is not UNSET:
            field_dict["instance"] = instance
        if extensions is not UNSET:
            field_dict["extensions"] = extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        title = d.pop("title", UNSET)

        status = d.pop("status", UNSET)

        detail = d.pop("detail", UNSET)

        instance = d.pop("instance", UNSET)

        _extensions = d.pop("extensions", UNSET)
        extensions: Union[Unset, ProblemDetailsExtensions]
        if isinstance(_extensions, Unset):
            extensions = UNSET
        else:
            extensions = ProblemDetailsExtensions.from_dict(_extensions)

        problem_details = cls(
            type=type,
            title=title,
            status=status,
            detail=detail,
            instance=instance,
            extensions=extensions,
        )

        problem_details.additional_properties = d
        return problem_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.problem_details import ProblemDetails
from ...types import Response


def _get_kwargs(
    workspace_id: str,
    data_point_id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/DataPoint/workspace/{workspaceId}/dataPoint/{dataPointId}".format(
        client.base_url, workspaceId=workspace_id, dataPointId=data_point_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "delete",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[ProblemDetails]:
    if response.status_code == 401:
        response_401 = ProblemDetails.from_dict(response.json())

        return response_401
    if response.status_code == 403:
        response_403 = ProblemDetails.from_dict(response.json())

        return response_403
    return None


def _build_response(*, response: httpx.Response) -> Response[ProblemDetails]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    workspace_id: str,
    data_point_id: str,
    *,
    client: Client,
) -> Response[ProblemDetails]:
    """
    Args:
        workspace_id (str):
        data_point_id (str):

    Returns:
        Response[ProblemDetails]
    """

    kwargs = _get_kwargs(
        workspace_id=workspace_id,
        data_point_id=data_point_id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    workspace_id: str,
    data_point_id: str,
    *,
    client: Client,
) -> Optional[ProblemDetails]:
    """
    Args:
        workspace_id (str):
        data_point_id (str):

    Returns:
        Response[ProblemDetails]
    """

    return sync_detailed(
        workspace_id=workspace_id,
        data_point_id=data_point_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    workspace_id: str,
    data_point_id: str,
    *,
    client: Client,
) -> Response[ProblemDetails]:
    """
    Args:
        workspace_id (str):
        data_point_id (str):

    Returns:
        Response[ProblemDetails]
    """

    kwargs = _get_kwargs(
        workspace_id=workspace_id,
        data_point_id=data_point_id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    workspace_id: str,
    data_point_id: str,
    *,
    client: Client,
) -> Optional[ProblemDetails]:
    """
    Args:
        workspace_id (str):
        data_point_id (str):

    Returns:
        Response[ProblemDetails]
    """

    return (
        await asyncio_detailed(
            workspace_id=workspace_id,
            data_point_id=data_point_id,
            client=client,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DayDataByHourTransfer")


@attr.s(auto_attribs=True)
class DayDataByHourTransfer:
    """
    Attributes:
        discriminator (str):
        h0 (Union[Unset, float]):
        h1 (Union[Unset, float]):
        h2 (Union[Unset, float]):
        h3 (Union[Unset, float]):
        h4 (Union[Unset, float]):
        h5 (Union[Unset, float]):
        h6 (Union[Unset, float]):
        h7 (Union[Unset, float]):
        h8 (Union[Unset, float]):
        h9 (Union[Unset, float]):
        h10 (Union[Unset, float]):
        h11 (Union[Unset, float]):
        h12 (Union[Unset, float]):
        h13 (Union[Unset, float]):
        h14 (Union[Unset, float]):
        h15 (Union[Unset, float]):
        h16 (Union[Unset, float]):
        h17 (Union[Unset, float]):
        h18 (Union[Unset, float]):
        h19 (Union[Unset, float]):
        h20 (Union[Unset, float]):
        h21 (Union[Unset, float]):
        h22 (Union[Unset, float]):
        h23 (Union[Unset, float]):
    """

    discriminator: str
    h0: Union[Unset, float] = UNSET
    h1: Union[Unset, float] = UNSET
    h2: Union[Unset, float] = UNSET
    h3: Union[Unset, float] = UNSET
    h4: Union[Unset, float] = UNSET
    h5: Union[Unset, float] = UNSET
    h6: Union[Unset, float] = UNSET
    h7: Union[Unset, float] = UNSET
    h8: Union[Unset, float] = UNSET
    h9: Union[Unset, float] = UNSET
    h10: Union[Unset, float] = UNSET
    h11: Union[Unset, float] = UNSET
    h12: Union[Unset, float] = UNSET
    h13: Union[Unset, float] = UNSET
    h14: Union[Unset, float] = UNSET
    h15: Union[Unset, float] = UNSET
    h16: Union[Unset, float] = UNSET
    h17: Union[Unset, float] = UNSET
    h18: Union[Unset, float] = UNSET
    h19: Union[Unset, float] = UNSET
    h20: Union[Unset, float] = UNSET
    h21: Union[Unset, float] = UNSET
    h22: Union[Unset, float] = UNSET
    h23: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        discriminator = self.discriminator
        h0 = self.h0
        h1 = self.h1
        h2 = self.h2
        h3 = self.h3
        h4 = self.h4
        h5 = self.h5
        h6 = self.h6
        h7 = self.h7
        h8 = self.h8
        h9 = self.h9
        h10 = self.h10
        h11 = self.h11
        h12 = self.h12
        h13 = self.h13
        h14 = self.h14
        h15 = self.h15
        h16 = self.h16
        h17 = self.h17
        h18 = self.h18
        h19 = self.h19
        h20 = self.h20
        h21 = self.h21
        h22 = self.h22
        h23 = self.h23

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "discriminator": discriminator,
            }
        )
        if h0 is not UNSET:
            field_dict["h0"] = h0
        if h1 is not UNSET:
            field_dict["h1"] = h1
        if h2 is not UNSET:
            field_dict["h2"] = h2
        if h3 is not UNSET:
            field_dict["h3"] = h3
        if h4 is not UNSET:
            field_dict["h4"] = h4
        if h5 is not UNSET:
            field_dict["h5"] = h5
        if h6 is not UNSET:
            field_dict["h6"] = h6
        if h7 is not UNSET:
            field_dict["h7"] = h7
        if h8 is not UNSET:
            field_dict["h8"] = h8
        if h9 is not UNSET:
            field_dict["h9"] = h9
        if h10 is not UNSET:
            field_dict["h10"] = h10
        if h11 is not UNSET:
            field_dict["h11"] = h11
        if h12 is not UNSET:
            field_dict["h12"] = h12
        if h13 is not UNSET:
            field_dict["h13"] = h13
        if h14 is not UNSET:
            field_dict["h14"] = h14
        if h15 is not UNSET:
            field_dict["h15"] = h15
        if h16 is not UNSET:
            field_dict["h16"] = h16
        if h17 is not UNSET:
            field_dict["h17"] = h17
        if h18 is not UNSET:
            field_dict["h18"] = h18
        if h19 is not UNSET:
            field_dict["h19"] = h19
        if h20 is not UNSET:
            field_dict["h20"] = h20
        if h21 is not UNSET:
            field_dict["h21"] = h21
        if h22 is not UNSET:
            field_dict["h22"] = h22
        if h23 is not UNSET:
            field_dict["h23"] = h23

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discriminator = d.pop("discriminator")

        h0 = d.pop("h0", UNSET)

        h1 = d.pop("h1", UNSET)

        h2 = d.pop("h2", UNSET)

        h3 = d.pop("h3", UNSET)

        h4 = d.pop("h4", UNSET)

        h5 = d.pop("h5", UNSET)

        h6 = d.pop("h6", UNSET)

        h7 = d.pop("h7", UNSET)

        h8 = d.pop("h8", UNSET)

        h9 = d.pop("h9", UNSET)

        h10 = d.pop("h10", UNSET)

        h11 = d.pop("h11", UNSET)

        h12 = d.pop("h12", UNSET)

        h13 = d.pop("h13", UNSET)

        h14 = d.pop("h14", UNSET)

        h15 = d.pop("h15", UNSET)

        h16 = d.pop("h16", UNSET)

        h17 = d.pop("h17", UNSET)

        h18 = d.pop("h18", UNSET)

        h19 = d.pop("h19", UNSET)

        h20 = d.pop("h20", UNSET)

        h21 = d.pop("h21", UNSET)

        h22 = d.pop("h22", UNSET)

        h23 = d.pop("h23", UNSET)

        day_data_by_hour_transfer = cls(
            discriminator=discriminator,
            h0=h0,
            h1=h1,
            h2=h2,
            h3=h3,
            h4=h4,
            h5=h5,
            h6=h6,
            h7=h7,
            h8=h8,
            h9=h9,
            h10=h10,
            h11=h11,
            h12=h12,
            h13=h13,
            h14=h14,
            h15=h15,
            h16=h16,
            h17=h17,
            h18=h18,
            h19=h19,
            h20=h20,
            h21=h21,
            h22=h22,
            h23=h23,
        )

        day_data_by_hour_transfer.additional_properties = d
        return day_data_by_hour_transfer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import Client
from ...models.en_data_point_existence_dto import EnDataPointExistenceDTO
from ...models.en_data_point_type import EnDataPointType
from ...models.problem_details import ProblemDetails
from ...types import UNSET, Response, Unset


def _get_kwargs(
    workspace_id: str,
    *,
    client: Client,
    filter_tags: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[EnDataPointType]] = UNSET,
    existence: Union[Unset, None, List[EnDataPointExistenceDTO]] = UNSET,
) -> Dict[str, Any]:
    url = "{}/DataPoint/workspace/{workspaceId}/tags".format(client.base_url, workspaceId=workspace_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_filter_tags: Union[Unset, None, List[str]] = UNSET
    if not isinstance(filter_tags, Unset):
        if filter_tags is None:
            json_filter_tags = None
        else:
            json_filter_tags = filter_tags

    params["filterTags"] = json_filter_tags

    json_type: Union[Unset, None, List[str]] = UNSET
    if not isinstance(type, Unset):
        if type is None:
            json_type = None
        else:
            json_type = []
            for type_item_data in type:
                type_item = type_item_data.value

                json_type.append(type_item)

    params["type"] = json_type

    json_existence: Union[Unset, None, List[str]] = UNSET
    if not isinstance(existence, Unset):
        if existence is None:
            json_existence = None
        else:
            json_existence = []
            for existence_item_data in existence:
                existence_item = existence_item_data.value

                json_existence.append(existence_item)

    params["existence"] = json_existence

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[List[str], ProblemDetails]]:
    if response.status_code == 200:
        response_200 = cast(List[str], response.json())

        return response_200
    if response.status_code == 401:
        response_401 = ProblemDetails.from_dict(response.json())

        return response_401
    if response.status_code == 403:
        response_403 = ProblemDetails.from_dict(response.json())

        return response_403
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[List[str], ProblemDetails]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    workspace_id: str,
    *,
    client: Client,
    filter_tags: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[EnDataPointType]] = UNSET,
    existence: Union[Unset, None, List[EnDataPointExistenceDTO]] = UNSET,
) -> Response[Union[List[str], ProblemDetails]]:
    """
    Args:
        workspace_id (str):
        filter_tags (Union[Unset, None, List[str]]):
        type (Union[Unset, None, List[EnDataPointType]]):
        existence (Union[Unset, None, List[EnDataPointExistenceDTO]]):

    Returns:
        Response[Union[List[str], ProblemDetails]]
    """

    kwargs = _get_kwargs(
        workspace_id=workspace_id,
        client=client,
        filter_tags=filter_tags,
        type=type,
        existence=existence,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    workspace_id: str,
    *,
    client: Client,
    filter_tags: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[EnDataPointType]] = UNSET,
    existence: Union[Unset, None, List[EnDataPointExistenceDTO]] = UNSET,
) -> Optional[Union[List[str], ProblemDetails]]:
    """
    Args:
        workspace_id (str):
        filter_tags (Union[Unset, None, List[str]]):
        type (Union[Unset, None, List[EnDataPointType]]):
        existence (Union[Unset, None, List[EnDataPointExistenceDTO]]):

    Returns:
        Response[Union[List[str], ProblemDetails]]
    """

    return sync_detailed(
        workspace_id=workspace_id,
        client=client,
        filter_tags=filter_tags,
        type=type,
        existence=existence,
    ).parsed


async def asyncio_detailed(
    workspace_id: str,
    *,
    client: Client,
    filter_tags: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[EnDataPointType]] = UNSET,
    existence: Union[Unset, None, List[EnDataPointExistenceDTO]] = UNSET,
) -> Response[Union[List[str], ProblemDetails]]:
    """
    Args:
        workspace_id (str):
        filter_tags (Union[Unset, None, List[str]]):
        type (Union[Unset, None, List[EnDataPointType]]):
        existence (Union[Unset, None, List[EnDataPointExistenceDTO]]):

    Returns:
        Response[Union[List[str], ProblemDetails]]
    """

    kwargs = _get_kwargs(
        workspace_id=workspace_id,
        client=client,
        filter_tags=filter_tags,
        type=type,
        existence=existence,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    workspace_id: str,
    *,
    client: Client,
    filter_tags: Union[Unset, None, List[str]] = UNSET,
    type: Union[Unset, None, List[EnDataPointType]] = UNSET,
    existence: Union[Unset, None, List[EnDataPointExistenceDTO]] = UNSET,
) -> Optional[Union[List[str], ProblemDetails]]:
    """
    Args:
        workspace_id (str):
        filter_tags (Union[Unset, None, List[str]]):
        type (Union[Unset, None, List[EnDataPointType]]):
        existence (Union[Unset, None, List[EnDataPointExistenceDTO]]):

    Returns:
        Response[Union[List[str], ProblemDetails]]
    """

    return (
        await asyncio_detailed(
            workspace_id=workspace_id,
            client=client,
            filter_tags=filter_tags,
            type=type,
            existence=existence,
        )
    ).parsed
